@@ -3,14 +3,11 @@
 
 #include <Rdefines.h>
 
-/* Copied from S4Arrays/src/array_selection.h */
-#define	INVALID_COORD(coord, maxcoord) \
-	((coord) == NA_INTEGER || (coord) < 1 || (coord) > (maxcoord))
-
 SEXP C_subassign_SVT_by_Lindex(
 	SEXP x_dim,
 	SEXP x_type,
 	SEXP x_SVT,
+	SEXP x_na_background,
 	SEXP Lindex,
 	SEXP vals
 );
@@ -35,7 +32,8 @@ SEXP C_subassign_SVT_with_Rarray(
 	SEXP x_dim,
 	SEXP x_type,
 	SEXP x_SVT,
-	SEXP Nindex,
+	SEXP x_na_background,
+	SEXP Noffs,
 	SEXP Rarray
 );
 
@@ -43,7 +41,7 @@ SEXP C_subassign_SVT_with_SVT(
 	SEXP x_dim,
 	SEXP x_type,
 	SEXP x_SVT,
-	SEXP Nindex,
+	SEXP Noffs,
 	SEXP v_dim,
 	SEXP v_type,
 	SEXP v_SVT
