@@ -38,6 +38,9 @@ int _get_Compare_opcode(SEXP op)
 
 /****************************************************************************
  * Core Compare_<Ltype>_<Rtype>() functions (10 in total)
+ *
+ * All these functions must return 'int0' (FALSE), 'int1' (TRUE),
+ * or 'intNA' (NA).
  */
 
 static inline int Compare_Rbyte_Rbyte(int opcode, Rbyte x, Rbyte y)
@@ -56,7 +59,7 @@ static inline int Compare_Rbyte_Rbyte(int opcode, Rbyte x, Rbyte y)
 }
 
 /* WARNING: Only valid to use on the int values of an integer vector (INTSXP).
-   Should NOT be used on the int values of a logical vector (LGLSEXP), on
+   Should NOT be used on the int values of a logical vector (LGLSXP), on
    which Compare_Rbyte_int() will generally produce wrong results!
    For example if Rbyte value 'x' is >= 2 and int value 'y' is 1 (TRUE),
    then == and <= will both return 0 when they are expected to return 1. */
@@ -65,7 +68,7 @@ static inline int Compare_Rbyte_int(int opcode, Rbyte x, int y)
 	int x1;
 
 	if (y == NA_INTEGER)
-		return NA_INTEGER;
+		return intNA;
 	x1 = (int) x;
 	switch (opcode) {
 	    case EQ_OPCODE: return x1 == y;
@@ -85,7 +88,7 @@ static inline int Compare_Rbyte_double(int opcode, Rbyte x, double y)
 	double x1;
 
 	if (ISNAN(y))
-		return NA_INTEGER;
+		return intNA;
 	x1 = (double) x;
 	switch (opcode) {
 	    case EQ_OPCODE: return x1 == y;
@@ -105,7 +108,7 @@ static inline int Compare_Rbyte_Rcomplex(int opcode, Rbyte x, Rcomplex y)
 	double x1;
 
 	if (ISNAN(y.r) || ISNAN(y.i))
-		return NA_INTEGER;
+		return intNA;
 	x1 = (double) x;
 	switch (opcode) {
 	    case EQ_OPCODE: return x1 == y.r && 0.0 == y.i;
@@ -119,7 +122,7 @@ static inline int Compare_Rbyte_Rcomplex(int opcode, Rbyte x, Rcomplex y)
 static inline int Compare_int_int(int opcode, int x, int y)
 {
 	if (x == NA_INTEGER || y == NA_INTEGER)
-		return NA_INTEGER;
+		return intNA;
 	switch (opcode) {
 	    case EQ_OPCODE: return x == y;
 	    case NE_OPCODE: return x != y;
@@ -138,7 +141,7 @@ static inline int Compare_int_double(int opcode, int x, double y)
 	double x1;
 
 	if (x == NA_INTEGER || ISNAN(y))
-		return NA_INTEGER;
+		return intNA;
 	x1 = (double) x;
 	switch (opcode) {
 	    case EQ_OPCODE: return x1 == y;
@@ -158,7 +161,7 @@ static inline int Compare_int_Rcomplex(int opcode, int x, Rcomplex y)
 	double x1;
 
 	if (x == NA_INTEGER || ISNAN(y.r) || ISNAN(y.i))
-		return NA_INTEGER;
+		return intNA;
 	x1 = (double) x;
 	switch (opcode) {
 	    case EQ_OPCODE: return x1 == y.r && 0.0 == y.i;
@@ -172,7 +175,7 @@ static inline int Compare_int_Rcomplex(int opcode, int x, Rcomplex y)
 static inline int Compare_double_double(int opcode, double x, double y)
 {
 	if (ISNAN(x) || ISNAN(y))
-		return NA_INTEGER;
+		return intNA;
 	switch (opcode) {
 	    case EQ_OPCODE: return x == y;
 	    case NE_OPCODE: return x != y;
@@ -189,7 +192,7 @@ static inline int Compare_double_double(int opcode, double x, double y)
 static inline int Compare_double_Rcomplex(int opcode, double x, Rcomplex y)
 {
 	if (ISNAN(x) || ISNAN(y.r) || ISNAN(y.i))
-		return NA_INTEGER;
+		return intNA;
 	switch (opcode) {
 	    case EQ_OPCODE: return x == y.r && 0.0 == y.i;
 	    case NE_OPCODE: return x != y.r || 0.0 != y.i;
@@ -202,7 +205,7 @@ static inline int Compare_double_Rcomplex(int opcode, double x, Rcomplex y)
 static inline int Compare_Rcomplex_Rcomplex(int opcode, Rcomplex x, Rcomplex y)
 {
 	if (ISNAN(x.r) || ISNAN(x.i) || ISNAN(y.r) || ISNAN(y.i))
-		return NA_INTEGER;
+		return intNA;
 	switch (opcode) {
 	    case EQ_OPCODE: return x.r == y.r && x.i == y.i;
 	    case NE_OPCODE: return x.r != y.r || x.i != y.i;
@@ -214,104 +217,135 @@ static inline int Compare_Rcomplex_Rcomplex(int opcode, Rcomplex x, Rcomplex y)
 
 
 /****************************************************************************
- * Two macros to generate the Compare_<Ltype>SV_<Rtype>()
- * and Compare_<Ltype>SV_<Rtype>SV() functions (20 in total)
+ * Two macros to generate the code (arg list + body) of the following
+ * functions:
+ *   - Compare_<Ltype>SV_<Rtype>() (10 functions)
+ *   - Compare_<Ltype>SV_<Rtype>SV() (10 functions)
  */
 
-/* Generate def of Compare_<Ltype>SV_<Rtype>() functions. */
+/* Generate code of Compare_<Ltype>SV_<Rtype>() functions.
+   These functions should never be called on 'sv1' and 'y' when 'sv1'
+   has a background set to zero and 'y' is NA or NaN. */
 #define FUNDEF_Compare_LtypeSV_Rtype(Ltype, Rtype)(int opcode,		\
 		const SparseVec *sv1, Rtype y,				\
-		int *out_nzvals, int *out_nzoffs)			\
+		SparseVec *out_sv)					\
 {									\
-	const Ltype *nzvals1 = get_ ## Ltype ## SV_nzvals(sv1);		\
-	int nzcount1 = get_SV_nzcount(sv1);				\
-	int out_nzcount = 0;						\
-	for (int k = 0; k < nzcount1; k++) {				\
-		Ltype x = nzvals1[k];					\
-		int v = Compare_ ## Ltype ## _ ## Rtype			\
-					(opcode, x, y);			\
-		if (v != 0) {						\
-			out_nzvals[out_nzcount] = v;			\
-			out_nzoffs[out_nzcount] = sv1->nzoffs[k];	\
-			out_nzcount++;					\
-		}							\
+        if (out_sv->len != sv1->len)					\
+                error("SparseArray internal error in "			\
+                      "Compare_<Ltype>SV_<Rtype>():\n"			\
+                      "    'sv1' and 'out_sv' are incompatible");	\
+	int *out_nzvals = (int *) out_sv->nzvals;			\
+	out_sv->nzcount = 0;						\
+	int out_bg_val = out_sv->na_background ? intNA : int0;		\
+	const Ltype *nzvals1_p = get_ ## Ltype ## SV_nzvals_p(sv1);	\
+	if (nzvals1_p == NULL) {  /* lacunar SparseVec */		\
+		int out_val = Compare_ ## Ltype ## _ ## Rtype		\
+					(opcode, Ltype ## 1, y);	\
+		if (out_val == out_bg_val)				\
+			return;						\
+		/* What 'out_val' is expected to be at this point    */	\
+		/* depends on 'out_bg_val':                          */	\
+		/* - If 'out_bg_val' is 'int0' then 'sv1' also       */	\
+		/*   has a background set to zero so 'y' cannot      */	\
+		/*   be NA or NaN (i.e. is.na(y) must be FALSE).     */	\
+		/*   This means that 'out_val' can only be TRUE      */	\
+		/*   (i.e. 'int1'). In particular 'out_val' cannot   */	\
+		/*   be NA (i.e. 'intNA') or FALSE (i.e. 'int0').    */	\
+		/* - If background is NA then 'out_val' can be TRUE  */	\
+		/*   or FALSE. It cannot be NA.                      */	\
+		out_nzvals[0] = out_val;				\
+		out_sv->nzcount = PROPAGATE_NZOFFS;			\
+		return;							\
 	}								\
-	return out_nzcount;						\
+	/* regular SparseVec */						\
+	int nzcount1 = get_SV_nzcount(sv1);				\
+	for (int k = 0; k < nzcount1; k++) {				\
+		int out_val = Compare_ ## Ltype ## _ ## Rtype		\
+					(opcode, nzvals1_p[k], y);	\
+		if (out_val == out_bg_val)				\
+			continue;					\
+		APPEND_TO_NZVALS_NZOFFS(out_val, sv1->nzoffs[k],	\
+			out_nzvals, out_sv->nzoffs, out_sv->nzcount);	\
+	}								\
+	return;								\
 }
 
-/* Generate def of Compare_<Ltype>SV_<Rtype>SV() functions. */
+/* Generate code of Compare_<Ltype>SV_<Rtype>SV() functions. */
 #define FUNDEF_Compare_LtypeSV_RtypeSV(Ltype, Rtype)(int opcode,	\
 		const SparseVec *sv1, const SparseVec *sv2,		\
-		int *out_nzvals, int *out_nzoffs)			\
+		SparseVec *out_sv)					\
 {									\
-	int k1, k2, off;						\
+        if (out_sv->len != sv1->len || out_sv->len != sv2->len)		\
+                error("SparseArray internal error in "			\
+                      "Compare_<Ltype>SV_<Rtype>SV()():\n"		\
+                      "    'sv1', 'sv2', and 'out_sv' are incompatible"); \
+	int *out_nzvals = (int *) out_sv->nzvals;			\
+	out_sv->nzcount = 0;						\
+	int out_bg_val = out_sv->na_background ? intNA : int0;		\
+	int k1 = 0, k2 = 0, off;					\
 	Ltype x;							\
 	Rtype y;							\
-									\
-	int out_nzcount = 0;						\
-	k1 = k2 = 0;							\
-	while (next_nzvals_ ## Ltype ## _ ## Rtype			\
+	while (next_ ## Ltype ## SV_ ## Rtype ## SV_vals		\
 		(sv1, sv2, &k1, &k2, &off, &x, &y))			\
 	{								\
-		int v = Compare_ ## Ltype ## _ ## Rtype			\
+		int out_val = Compare_ ## Ltype ## _ ## Rtype		\
 					(opcode, x, y);			\
-		if (v != 0) {						\
-			out_nzvals[out_nzcount] = v;			\
-			out_nzoffs[out_nzcount] = off;			\
-			out_nzcount++;					\
-		}							\
+		if (out_val == out_bg_val)				\
+			continue;					\
+		APPEND_TO_NZVALS_NZOFFS(out_val, off,			\
+			out_nzvals, out_sv->nzoffs, out_sv->nzcount);   \
 	}								\
-	return out_nzcount;						\
+	return;								\
 }
 
-static int Compare_RbyteSV_Rbyte
+static void Compare_RbyteSV_Rbyte
 	FUNDEF_Compare_LtypeSV_Rtype(Rbyte, Rbyte)
-static int Compare_RbyteSV_RbyteSV
+static void Compare_RbyteSV_RbyteSV
 	FUNDEF_Compare_LtypeSV_RtypeSV(Rbyte, Rbyte)
 
-static int Compare_RbyteSV_int
+static void Compare_RbyteSV_int
 	FUNDEF_Compare_LtypeSV_Rtype(Rbyte, int)
-static int Compare_RbyteSV_intSV
+static void Compare_RbyteSV_intSV
 	FUNDEF_Compare_LtypeSV_RtypeSV(Rbyte, int)
 
-static int Compare_RbytesSV_double
+static void Compare_RbytesSV_double
 	FUNDEF_Compare_LtypeSV_Rtype(Rbyte, double)
-static int Compare_RbytesSV_doubleSV
+static void Compare_RbytesSV_doubleSV
 	FUNDEF_Compare_LtypeSV_RtypeSV(Rbyte, double)
 
-static int Compare_RbyteSV_Rcomplex
+static void Compare_RbyteSV_Rcomplex
 	FUNDEF_Compare_LtypeSV_Rtype(Rbyte, Rcomplex)
-static int Compare_RbyteSV_RcomplexSV
+static void Compare_RbyteSV_RcomplexSV
 	FUNDEF_Compare_LtypeSV_RtypeSV(Rbyte, Rcomplex)
 
-static int Compare_intSV_int
+static void Compare_intSV_int
 	FUNDEF_Compare_LtypeSV_Rtype(int, int)
-static int Compare_intSV_intSV
+static void Compare_intSV_intSV
 	FUNDEF_Compare_LtypeSV_RtypeSV(int, int)
 
-static int Compare_intSV_double
+static void Compare_intSV_double
 	FUNDEF_Compare_LtypeSV_Rtype(int, double)
-static int Compare_intSV_doubleSV
+static void Compare_intSV_doubleSV
 	FUNDEF_Compare_LtypeSV_RtypeSV(int, double)
 
-static int Compare_intSV_Rcomplex
+static void Compare_intSV_Rcomplex
 	FUNDEF_Compare_LtypeSV_Rtype(int, Rcomplex)
-static int Compare_intSV_RcomplexSV
+static void Compare_intSV_RcomplexSV
 	FUNDEF_Compare_LtypeSV_RtypeSV(int, Rcomplex)
 
-static int Compare_doubleSV_double
+static void Compare_doubleSV_double
 	FUNDEF_Compare_LtypeSV_Rtype(double, double)
-static int Compare_doubleSV_doubleSV
+static void Compare_doubleSV_doubleSV
 	FUNDEF_Compare_LtypeSV_RtypeSV(double, double)
 
-static int Compare_doubleSV_Rcomplex
+static void Compare_doubleSV_Rcomplex
 	FUNDEF_Compare_LtypeSV_Rtype(double, Rcomplex)
-static int Compare_doubleSV_RcomplexSV
+static void Compare_doubleSV_RcomplexSV
 	FUNDEF_Compare_LtypeSV_RtypeSV(double, Rcomplex)
 
-static int Compare_RcomplexSV_Rcomplex
+static void Compare_RcomplexSV_Rcomplex
 	FUNDEF_Compare_LtypeSV_Rtype(Rcomplex, Rcomplex)
-static int Compare_RcomplexSV_RcomplexSV
+static void Compare_RcomplexSV_RcomplexSV
 	FUNDEF_Compare_LtypeSV_RtypeSV(Rcomplex, Rcomplex)
 
 
@@ -326,99 +360,91 @@ static int Compare_RcomplexSV_RcomplexSV
  */
 
 /* Type of 'scalar' is expected to be "raw" or bigger. */
-static int Compare_RbyteSV_scalar(int opcode,
-		const SparseVec *sv1, SEXP scalar,
-		int *out_nzvals, int *out_nzoffs)
+static void Compare_RbyteSV_scalar(int opcode,
+		const SparseVec *sv1, SEXP scalar, SparseVec *out_sv)
 {
 	SEXPTYPE Rtype2 = TYPEOF(scalar);
 	switch (Rtype2) {
 	    case RAWSXP:
-		return Compare_RbyteSV_Rbyte(opcode,
-					sv1, RAW(scalar)[0],
-					out_nzvals, out_nzoffs);
-	    case LGLSXP: case INTSXP:
-		return Compare_RbyteSV_int(opcode,
-					sv1, INTEGER(scalar)[0],
-					out_nzvals, out_nzoffs);
+		Compare_RbyteSV_Rbyte(opcode,
+				sv1, RAW(scalar)[0], out_sv);
+		return;
+	    case INTSXP: case LGLSXP:
+		Compare_RbyteSV_int(opcode,
+				sv1, INTEGER(scalar)[0], out_sv);
+		return;
 	    case REALSXP:
-		return Compare_RbytesSV_double(opcode,
-					sv1, REAL(scalar)[0],
-					out_nzvals, out_nzoffs);
+		Compare_RbytesSV_double(opcode,
+				sv1, REAL(scalar)[0], out_sv);
+		return;
 	    case CPLXSXP:
-		return Compare_RbyteSV_Rcomplex(opcode,
-					sv1, COMPLEX(scalar)[0],
-					out_nzvals, out_nzoffs);
+		Compare_RbyteSV_Rcomplex(opcode,
+				sv1, COMPLEX(scalar)[0], out_sv);
+		return;
 	}
 	error("SparseArray internal error in "
 	      "Compare_RbyteSV_scalar():\n"
 	      "    unsupported 'Rtype2': \"%s\"", type2char(Rtype2));
-	return 0;  /* will never reach this */
 }
 
 /* Type of 'scalar' is expected to be "integer" or bigger. */
-static int Compare_intSV_scalar(int opcode,
-		const SparseVec *sv1, SEXP scalar,
-		int *out_nzvals, int *out_nzoffs)
+static void Compare_intSV_scalar(int opcode,
+		const SparseVec *sv1, SEXP scalar, SparseVec *out_sv)
 {
 	SEXPTYPE Rtype2 = TYPEOF(scalar);
 	switch (Rtype2) {
-	    case LGLSXP: case INTSXP:
-		return Compare_intSV_int(opcode,
-					sv1, INTEGER(scalar)[0],
-					out_nzvals, out_nzoffs);
+	    case INTSXP: case LGLSXP:
+		Compare_intSV_int(opcode,
+				sv1, INTEGER(scalar)[0], out_sv);
+		return;
 	    case REALSXP:
-		return Compare_intSV_double(opcode,
-					sv1, REAL(scalar)[0],
-					out_nzvals, out_nzoffs);
+		Compare_intSV_double(opcode,
+				sv1, REAL(scalar)[0], out_sv);
+		return;
 	    case CPLXSXP:
-		return Compare_intSV_Rcomplex(opcode,
-					sv1, COMPLEX(scalar)[0],
-					out_nzvals, out_nzoffs);
+		Compare_intSV_Rcomplex(opcode,
+				sv1, COMPLEX(scalar)[0], out_sv);
+		return;
 	}
 	error("SparseArray internal error in "
 	      "Compare_intSV_scalar():\n"
 	      "    unsupported 'Rtype2': \"%s\"", type2char(Rtype2));
-	return 0;  /* will never reach this */
 }
 
 /* Type of 'scalar' is expected to be "double" or bigger. */
-static int Compare_doubleSV_scalar(int opcode,
-		const SparseVec *sv1, SEXP scalar,
-		int *out_nzvals, int *out_nzoffs)
+static void Compare_doubleSV_scalar(int opcode,
+		const SparseVec *sv1, SEXP scalar, SparseVec *out_sv)
 {
 	SEXPTYPE Rtype2 = TYPEOF(scalar);
 	switch (Rtype2) {
 	    case REALSXP:
-		return Compare_doubleSV_double(opcode,
-					sv1, REAL(scalar)[0],
-					out_nzvals, out_nzoffs);
+		Compare_doubleSV_double(opcode,
+				sv1, REAL(scalar)[0], out_sv);
+		return;
 	    case CPLXSXP:
-		return Compare_doubleSV_Rcomplex(opcode,
-					sv1, COMPLEX(scalar)[0],
-					out_nzvals, out_nzoffs);
+		Compare_doubleSV_Rcomplex(opcode,
+				sv1, COMPLEX(scalar)[0], out_sv);
+		return;
 	}
 	error("SparseArray internal error in "
 	      "Compare_doubleSV_scalar():\n"
 	      "    unsupported 'Rtype2': \"%s\"", type2char(Rtype2));
-	return 0;  /* will never reach this */
 }
 
 /* Type of 'scalar' is expected to be "complex". */
-static int Compare_RcomplexSV_scalar(int opcode,
-		const SparseVec *sv1, SEXP scalar,
-		int *out_nzvals, int *out_nzoffs)
+static void Compare_RcomplexSV_scalar(int opcode,
+		const SparseVec *sv1, SEXP scalar, SparseVec *out_sv)
 {
 	SEXPTYPE Rtype2 = TYPEOF(scalar);
 	switch (Rtype2) {
 	    case CPLXSXP:
-		return Compare_RcomplexSV_Rcomplex(opcode,
-					sv1, COMPLEX(scalar)[0],
-					out_nzvals, out_nzoffs);
+		Compare_RcomplexSV_Rcomplex(opcode, sv1, COMPLEX(scalar)[0],
+					out_sv);
+		return;
 	}
 	error("SparseArray internal error in "
 	      "Compare_RcomplexSV_scalar():\n"
 	      "    unsupported 'Rtype2': \"%s\"", type2char(Rtype2));
-	return 0;  /* will never reach this */
 }
 
 
@@ -429,104 +455,106 @@ static int Compare_RcomplexSV_scalar(int opcode,
  * Compare_RcomplexSV_SV()
  */
 
-static int Compare_RbyteSV_SV(int opcode,
+static void Compare_RbyteSV_SV(int opcode,
 		const SparseVec *sv1, const SparseVec *sv2,
-		int *out_nzvals, int *out_nzoffs)
+		SparseVec *out_sv)
 {
 	SEXPTYPE Rtype2 = get_SV_Rtype(sv2);
 	switch (Rtype2) {
 	    case RAWSXP:
-		return Compare_RbyteSV_RbyteSV(opcode,
-				sv1, sv2, out_nzvals, out_nzoffs);
-	    case LGLSXP: case INTSXP:
-		return Compare_RbyteSV_intSV(opcode,
-				sv1, sv2, out_nzvals, out_nzoffs);
+		Compare_RbyteSV_RbyteSV(opcode, sv1, sv2, out_sv);
+		return;
+	    case INTSXP: case LGLSXP:
+		Compare_RbyteSV_intSV(opcode, sv1, sv2, out_sv);
+		return;
 	    case REALSXP:
-		return Compare_RbytesSV_doubleSV(opcode,
-				sv1, sv2, out_nzvals, out_nzoffs);
+		Compare_RbytesSV_doubleSV(opcode, sv1, sv2, out_sv);
+		return;
 	    case CPLXSXP:
-		return Compare_RbyteSV_RcomplexSV(opcode,
-				sv1, sv2, out_nzvals, out_nzoffs);
+		Compare_RbyteSV_RcomplexSV(opcode, sv1, sv2, out_sv);
+		return;
 	}
 	error("SparseArray internal error in "
 	      "Compare_RbyteSV_SV():\n"
 	      "    unsupported 'Rtype2': \"%s\"", type2char(Rtype2));
-	return 0;  /* will never reach this */
 }
 
-static int Compare_intSV_SV(int opcode,
+static void Compare_intSV_SV(int opcode,
 		const SparseVec *sv1, const SparseVec *sv2,
-		int *out_nzvals, int *out_nzoffs)
+		SparseVec *out_sv)
 {
 	SEXPTYPE Rtype2 = get_SV_Rtype(sv2);
 	switch (Rtype2) {
 	    case RAWSXP:
-		return Compare_RbyteSV_intSV(flip_opcode(opcode),
-				sv2, sv1, out_nzvals, out_nzoffs);
-	    case LGLSXP: case INTSXP:
-		return Compare_intSV_intSV(opcode,
-				sv1, sv2, out_nzvals, out_nzoffs);
+		Compare_RbyteSV_intSV(flip_Compare_opcode(opcode),
+				sv2, sv1, out_sv);
+		return;
+	    case INTSXP: case LGLSXP:
+		Compare_intSV_intSV(opcode, sv1, sv2, out_sv);
+		return;
 	    case REALSXP:
-		return Compare_intSV_doubleSV(opcode,
-				sv1, sv2, out_nzvals, out_nzoffs);
+		Compare_intSV_doubleSV(opcode, sv1, sv2, out_sv);
+		return;
 	    case CPLXSXP:
-		return Compare_intSV_RcomplexSV(opcode,
-				sv1, sv2, out_nzvals, out_nzoffs);
+		Compare_intSV_RcomplexSV(opcode, sv1, sv2, out_sv);
+		return;
 	}
 	error("SparseArray internal error in "
 	      "Compare_intSV_SV():\n"
 	      "    unsupported 'Rtype2': \"%s\"", type2char(Rtype2));
-	return 0;  /* will never reach this */
 }
 
-static int Compare_doubleSV_SV(int opcode,
+static void Compare_doubleSV_SV(int opcode,
 		const SparseVec *sv1, const SparseVec *sv2,
-		int *out_nzvals, int *out_nzoffs)
+		SparseVec *out_sv)
 {
 	SEXPTYPE Rtype2 = get_SV_Rtype(sv2);
 	switch (Rtype2) {
 	    case RAWSXP:
-		return Compare_RbytesSV_doubleSV(flip_opcode(opcode),
-				sv2, sv1, out_nzvals, out_nzoffs);
-	    case LGLSXP: case INTSXP:
-		return Compare_intSV_doubleSV(flip_opcode(opcode),
-				sv2, sv1, out_nzvals, out_nzoffs);
+		Compare_RbytesSV_doubleSV(flip_Compare_opcode(opcode),
+				sv2, sv1, out_sv);
+		return;
+	    case INTSXP: case LGLSXP:
+		Compare_intSV_doubleSV(flip_Compare_opcode(opcode),
+				sv2, sv1, out_sv);
+		return;
 	    case REALSXP:
-		return Compare_doubleSV_doubleSV(opcode,
-				sv1, sv2, out_nzvals, out_nzoffs);
+		Compare_doubleSV_doubleSV(opcode, sv1, sv2, out_sv);
+		return;
 	    case CPLXSXP:
-		return Compare_doubleSV_RcomplexSV(opcode,
-				sv1, sv2, out_nzvals, out_nzoffs);
+		Compare_doubleSV_RcomplexSV(opcode, sv1, sv2, out_sv);
+		return;
 	}
 	error("SparseArray internal error in "
 	      "Compare_doubleSV_SV():\n"
 	      "    unsupported 'Rtype2': \"%s\"", type2char(Rtype2));
-	return 0;  /* will never reach this */
 }
 
-static int Compare_RcomplexSV_SV(int opcode,
+static void Compare_RcomplexSV_SV(int opcode,
 		const SparseVec *sv1, const SparseVec *sv2,
-		int *out_nzvals, int *out_nzoffs)
+		SparseVec *out_sv)
 {
 	SEXPTYPE Rtype2 = get_SV_Rtype(sv2);
 	switch (Rtype2) {
 	    case RAWSXP:
-		return Compare_RbyteSV_RcomplexSV(flip_opcode(opcode),
-				sv2, sv1, out_nzvals, out_nzoffs);
-	    case LGLSXP: case INTSXP:
-		return Compare_intSV_RcomplexSV(flip_opcode(opcode),
-				sv2, sv1, out_nzvals, out_nzoffs);
+		Compare_RbyteSV_RcomplexSV(flip_Compare_opcode(opcode),
+				sv2, sv1, out_sv);
+		return;
+	    case INTSXP: case LGLSXP:
+		Compare_intSV_RcomplexSV(flip_Compare_opcode(opcode),
+				sv2, sv1, out_sv);
+		return;
 	    case REALSXP:
-		return Compare_doubleSV_RcomplexSV(flip_opcode(opcode),
-				sv2, sv1, out_nzvals, out_nzoffs);
+		Compare_doubleSV_RcomplexSV(flip_Compare_opcode(opcode),
+				sv2, sv1, out_sv);
+		return;
 	    case CPLXSXP:
-		return Compare_RcomplexSV_RcomplexSV(opcode,
-				sv1, sv2, out_nzvals, out_nzoffs);
+		Compare_RcomplexSV_RcomplexSV(opcode, sv1, sv2, out_sv);
+		return;
 	}
 	error("SparseArray internal error in "
 	      "Compare_RcomplexSV_SV():\n"
 	      "    unsupported 'Rtype2': \"%s\"", type2char(Rtype2));
-	return 0;  /* will never reach this */
 }
 
 
@@ -536,77 +564,74 @@ static int Compare_RcomplexSV_SV(int opcode,
  * _Compare_sv1_sv2()
  */
 
-int _Compare_sv1_zero(int opcode, const SparseVec *sv1,
-		int *out_nzvals, int *out_nzoffs)
+void _Compare_sv1_zero(int opcode, const SparseVec *sv1,
+		SparseVec *out_sv)
 {
 	SEXPTYPE Rtype1 = get_SV_Rtype(sv1);
 	switch (Rtype1) {
 	    case RAWSXP:
-		return Compare_RbyteSV_Rbyte(opcode,
-				sv1, Rbyte0, out_nzvals, out_nzoffs);
-	    case LGLSXP: case INTSXP:
-		return Compare_intSV_int(opcode,
-				sv1, int0, out_nzvals, out_nzoffs);
+		Compare_RbyteSV_Rbyte(opcode, sv1, Rbyte0, out_sv);
+		return;
+	    case INTSXP: case LGLSXP:
+		Compare_intSV_int(opcode, sv1, int0, out_sv);
+		return;
 	    case REALSXP:
-		return Compare_doubleSV_double(opcode,
-				sv1, double0, out_nzvals, out_nzoffs);
+		Compare_doubleSV_double(opcode, sv1, double0, out_sv);
+		return;
 	    case CPLXSXP:
-		return Compare_RcomplexSV_Rcomplex(opcode,
-				sv1, Rcomplex0, out_nzvals, out_nzoffs);
+		Compare_RcomplexSV_Rcomplex(opcode, sv1, Rcomplex0, out_sv);
+		return;
 	}
 	error("SparseArray internal error in "
 	      "_Compare_sv1_zero():\n"
 	      "    unsupported 'Rtype1': \"%s\"", type2char(Rtype1));
-	return 0;  /* will never reach this */
 }
 
 /* 'scalar' is assumed to be an atomic vector of length 1. This is NOT checked!
-   Also its type is expected to be the same size as sv1's type or bigger. */
-int _Compare_sv1_scalar(int opcode, const SparseVec *sv1, SEXP scalar,
-		int *out_nzvals, int *out_nzoffs)
+   Also its type is expected to be the same as sv1's type or bigger. */
+void _Compare_sv1_scalar(int opcode, const SparseVec *sv1, SEXP scalar,
+		SparseVec *out_sv)
 {
 	SEXPTYPE Rtype1 = get_SV_Rtype(sv1);
 	switch (Rtype1) {
 	    case RAWSXP:
-		return Compare_RbyteSV_scalar(opcode,
-				sv1, scalar, out_nzvals, out_nzoffs);
-	    case LGLSXP: case INTSXP:
-		return Compare_intSV_scalar(opcode,
-				sv1, scalar, out_nzvals, out_nzoffs);
+		Compare_RbyteSV_scalar(opcode, sv1, scalar, out_sv);
+		return;
+	    case INTSXP: case LGLSXP:
+		Compare_intSV_scalar(opcode, sv1, scalar, out_sv);
+		return;
 	    case REALSXP:
-		return Compare_doubleSV_scalar(opcode,
-				sv1, scalar, out_nzvals, out_nzoffs);
+		Compare_doubleSV_scalar(opcode, sv1, scalar, out_sv);
+		return;
 	    case CPLXSXP:
-		return Compare_RcomplexSV_scalar(opcode,
-				sv1, scalar, out_nzvals, out_nzoffs);
+		Compare_RcomplexSV_scalar(opcode, sv1, scalar, out_sv);
+		return;
 	}
 	error("SparseArray internal error in "
 	      "_Compare_sv1_scalar():\n"
 	      "    unsupported 'Rtype1': \"%s\"", type2char(Rtype1));
-	return 0;  /* will never reach this */
 }
 
-int _Compare_sv1_sv2(int opcode, const SparseVec *sv1, const SparseVec *sv2,
-		int *out_nzvals, int *out_nzoffs)
+void _Compare_sv1_sv2(int opcode, const SparseVec *sv1, const SparseVec *sv2,
+		SparseVec *out_sv)
 {
 	SEXPTYPE Rtype1 = get_SV_Rtype(sv1);
 	switch (Rtype1) {
 	    case RAWSXP:
-		return Compare_RbyteSV_SV(opcode,
-				sv1, sv2, out_nzvals, out_nzoffs);
-	    case LGLSXP: case INTSXP:
-		return Compare_intSV_SV(opcode,
-				sv1, sv2, out_nzvals, out_nzoffs);
+		Compare_RbyteSV_SV(opcode, sv1, sv2, out_sv);
+		return;
+	    case INTSXP: case LGLSXP:
+		Compare_intSV_SV(opcode, sv1, sv2, out_sv);
+		return;
 	    case REALSXP:
-		return Compare_doubleSV_SV(opcode,
-				sv1, sv2, out_nzvals, out_nzoffs);
+		Compare_doubleSV_SV(opcode, sv1, sv2, out_sv);
+		return;
 	    case CPLXSXP:
-		return Compare_RcomplexSV_SV(opcode,
-				sv1, sv2, out_nzvals, out_nzoffs);
+		Compare_RcomplexSV_SV(opcode, sv1, sv2, out_sv);
+		return;
 	}
 	error("SparseArray internal error in "
 	      "_Compare_sv1_sv2():\n"
 	      "    unsupported 'Rtype1': \"%s\"", type2char(Rtype1));
-	return 0;  /* will never reach this */
 }
 
