@@ -1,9 +1,13 @@
 /****************************************************************************
- *               Wrappers to OpenMP thread control functions                *
+ *               Wrappers for OpenMP thread control functions               *
  ****************************************************************************/
 #include "thread_control.h"
 
 #ifdef _OPENMP
+/* <Rinternals.h> defines macro match that seems to break <omp.h> with
+   some versions of Clang.
+   See https://github.com/Bioconductor/SparseArray/issues/9 */
+#undef match
 #include <omp.h>
 #endif
 
